#ifndef _OME_CRYPTO_H
//! \cond
#define _OME_CRYPTO_H "$Id: OMEcrypto.h 446 2020-07-23 20:08:31Z geoff $"
//! \endcond

/** \defgroup OMEcrypto OME Cryptographic Routines
 * @{
 */

#include <OMEtype.h>

enum {
    OME_INIT_VEC_LEN_IN_BITS=128,
    OME_INIT_VEC_LEN_IN_HEX_BYTES=(128 / 4),
    OME_SHA1_HASH_LEN_IN_BITS=160,
    OME_SHA1_HASH_LEN_IN_BYTES=(160 / 8)
};

/*!
\brief Generate a random sequence of bits.
*/
extern OMEstring *OMEmakeRandomKey(int32_t bits = 1024);

/*!
\brief Compute Secure Hash Algorithm 1 over an OMEstring.
*/
extern OMEstring *OMEmakeSHA1hash(const OMEstring &message);

/*!
\brief Compute Secure Hash Algorithm 256 over an OMEstring.
*/
extern OMEstring *OMEmakeSHA256hash(const OMEstring &message);

/*!
\brief Compute Message Digest 5 over an OMEstring.
*/
extern OMEstring *OMEmakeMD5hash(const OMEstring &message);

extern OMEstring *OMEmakePrivateKey(const OMEstring &secretPhrase);

extern OMEstring *OMEmakePublicKey(const OMEstring &privateKey);

extern OMEstring *OMEmakeSessionKeyFromPublicKey(const OMEstring &publicKey,
    OMEstring &randomDataInSessionKeyOut);

extern OMEstring *OMEdecryptSessionKeyWithPrivateKey(const OMEstring &privateKey,
    const OMEstring &data);

extern int OMEinitNewCipher(const OMEstring &initVector);

extern int OMEfreeCipher(int useCipher);

extern int OMEdefineNewEncryptKey(const OMEstring &key);

extern int OMEdefineNewDecryptKey(const OMEstring &key);

extern int OMEfreeEncryptionKey(int useKey);

extern int OMEfreeDecryptionKey(int useKey);

extern int OMEencryptString(int useCipher, int useKey, const OMEstring &input,
    OMEstring &result);

extern int OMEdecryptString(int useCipher, int useKey, const OMEstring &input,
    OMEstring &result);

extern OMEstring *OMEreadLicenseFile(OMEstring *retFileName, const OMEstring &fileSuffix, const char *searchPath);

extern OMEtype *OMEgetLicenseAttribute(const OMEstring &attrName, OMEstring *retFileName = 0);

extern OMEtype *OMEgetLicenseAttribute(const char *attrName, OMEstring *retFileName = 0);

extern OMEtype *OMEgetSigningFile(const OMEstring &secret, const OMEstring &logicalDomain, bool isPublic = false);

enum {
    OME_ERROR_CRYPTO_BAD_VERSION=1,
    OME_ERROR_CRYPTO_BAD_MAGIC=2,
    OME_ERROR_CRYPTO_INTEGRITY1=3,
    OME_ERROR_CRYPTO_DECODE_FAILED=4,
    OME_ERROR_CRYPTO_DECODE_ARRAY=5,
    OME_ERROR_CRYPTO_BAD_DATA=6,
    OME_ERROR_CRYPTO_INIT_FAILED=7,
    OME_ERROR_CRYPTO_WRONG_HOST=8,
    OME_ERROR_CRYPTO_WRONG_USER=9,
    OME_ERROR_CRYPTO_INTEGRITY2=10
};

/** @}
 */

#endif
/* vim: set expandtab shiftwidth=4 tabstop=4: */
