#ifndef _OME_FILE_DESCRIPTOR_H
//! \cond
#define _OME_FILE_DESCRIPTOR_H "$Id: OMEfileDescriptor.h 357 2020-06-18 23:13:25Z geoff $"
//! \endcond

#include <OMEioObjects.h>

/** \addtogroup OMEio OME Internal I/O Classes
 * @{
 */

class OMEfileDescriptor final : public OMEioDescriptor {
private:
static OMEioDescriptor *openFile(const char *name, uint32_t openMode,
		int *errRC = nullptr);

	OMEstring	localName;
	bool		fileIsOpen;
public:
static OMEioDescriptor *openFileScheme(const OMEstring &url, int *errRet = nullptr);

	OMEfileDescriptor(uint64_t handle, OMEioEvent::OMEfileType type,
		const OMEstring *fileName);

	~OMEfileDescriptor();

	virtual int readBytes(unsigned char *bfr, size_t bytes, size_t *actualBytes, int *errRet) override;

	virtual int writeBytes(const unsigned char *bfr, size_t bytes, size_t *actualBytes, int *errRet) override;

#ifndef _WIN32
	virtual int writeVectorOfBytes(const OMEioVector *v, uint_fast16_t elementCount,
		size_t *actualBytes, int *errRet, size_t skipBytes = 0) override;
#endif
	virtual OMEevent *selectForRead(OMEeventCallbackFP func, OMEthread *t = nullptr) override;

	virtual OMEevent *selectForWrite(OMEeventCallbackFP func, OMEthread *t = nullptr) override;

	virtual OMEioDescriptor *acceptConnection(int *errRet) override;

	virtual int seekToOffset(size_t offset) override;

	virtual int close(int mask) override;

	virtual OMEstring *getLocalAddress() const override;

	virtual bool isOpen() const override { return (fileIsOpen); }
}; // end class OMEfileDescriptor

/** @}
 */
#endif
/* vim: set expandtab shiftwidth=4 tabstop=4: */
