#ifndef _OME_FUNCTIONS_H
//! \cond
#define _OME_FUNCTIONS_H "$Id: OMEfunctions.h 445 2020-07-23 20:07:15Z geoff $"
//! \endcond


#    include <OMEtype.h>

/*! \file
 * \brief OME utility functions.
 */

/** \defgroup OMEutilFunctions OME Utility Functions
 *
 * \brief Utility functions for the FARGOS/VISTA Object Management Environment
 * and attached applications.
 * @{
 */

OME_DLL_EXPORT uint32_t OMEhash(const unsigned char *data, const uint32_t len) NONNULL_PARAMETERS(1);

OME_DLL_EXPORT uint32_t OMEhash(const OMEstring &data);

OME_DLL_EXPORT uint32_t OMEmethodSignature(const int argc, const uint32_t types[],
        uint32_t *anyMask = nullptr);

OME_DLL_EXPORT OMEstring *OMEfindFileInPathSpec(const OMEstring &fileName,
        const OMEstring &path);

OME_DLL_EXPORT OMEstring *OMEfindFileInDirectory(const OMEstring &fileName,
        const OMEarray &dirList);

OME_DLL_EXPORT OMEarray *OMEparsePathSpecification(const OMEstring &pathSpec);

OME_DLL_EXPORT OMEarray *OMElistDirectory(const OMEstring &dirName, int *errRet = nullptr);

OME_DLL_EXPORT int OMEconvertLocalRelativeTimeToAbsolute(OMEtype &result,
        uint32_t tArg, int asGMT);

OME_DLL_EXPORT uint32_t OMEconvertAbsoluteToLocalRelativeTime(const OMEtype &tData);

OME_DLL_EXPORT int OMErfc1123Date(OMEtype &result, const OMEtype &argData);

OME_DLL_EXPORT int OMEiso8601Date(OMEtype &result, const OMEtype &argData);

OME_DLL_EXPORT int OMEconvertRFC1123date(OMEtype &result, const OMEtype &timeString);

OME_DLL_EXPORT int OMEtimeDifference(const OMEtype &time1, const OMEtype &time2);

OME_DLL_EXPORT int OMEtransportToSockaddr(union sockBigEnough *sock,
        uint_fast16_t *sockLen, int *sockMode,
        const char *url, uint_fast16_t urlLen) NONNULL_PARAMETERS(1, 2, 3, 4);

OME_DLL_EXPORT OMEstring *OMEdisplayableAddress(const struct sockaddr *sock,
        const int len, const uint32_t openMode) NONNULL_PARAMETERS(1);

OME_DLL_EXPORT int OMEgetInterfaces(OMEtype &result);

/** @}
 */

#endif
/* vim: set expandtab shiftwidth=4 tabstop=4: */
