#ifndef _OME_IO_OBJECTS_H
//! \cond
#define _OME_IO_OBJECTS_H "$Id: OMEioObjects.h 357 2020-06-18 23:13:25Z geoff $"
//! \endcond

#ifdef _WIN32
#include <winsock2.h>
#endif

#include <OMEioEvents.h>

/** \defgroup OMEio OME Internal I/O Classes
 * @{
 */

/*! \brief Equivalent of iov structure.
 */
struct OMEioVector {
	unsigned char	*data;
	size_t          length;
};


/*! \brief Base class for open files and sockets.
 */
class OMEioDescriptor {
public:
enum OMEopenMode { read=OME_IO_READ, write=OME_IO_WRITE, listen=OME_IO_LISTEN,
	connect=OME_IO_CONNECT, create=OME_IO_CREATE,
	mustExist=OME_IO_MUST_EXIST, truncate=OME_IO_TRUNCATE,
	append=OME_IO_APPEND, nonblocking=OME_IO_NONBLOCKING,
	datagram=OME_IO_DATAGRAM, raw=OME_IO_RAW, multicast=OME_IO_MULTICAST };

static OMEioDescriptor *openIOscheme(const char *name, uint32_t openMode);

	uint64_t			ioHandle;
	OMEioEvent::OMEfileType	descriptorType;

	OMEioDescriptor(uint64_t handle, OMEioEvent::OMEfileType type);

	virtual ~OMEioDescriptor();

#ifdef _WIN32
    /* this only needs to be virtual on Windows to deal with
     * WSAEVENT being the actual selectable object, rather than
     * the file descriptor.
     */
    virtual uint64_t getSelectHandle() const
#else
    uint64_t getSelectHandle() const OME_ALWAYS_INLINE
#endif
    {
        return (ioHandle);
    }

	virtual	int readBytes(unsigned char *bfr, size_t bytes, size_t *actualBytes,
		int *errRet) = 0;

	virtual	int writeBytes(const unsigned char *bfr, size_t bytes, size_t *actualBytes,
		int *errRet) = 0;

	// implemented as a sequence of writeBytes for subclass that can't
	// support the writeVectorOfBytes functionality..
	virtual int writeVectorOfBytes(const OMEioVector *v,
        uint_fast16_t elementCount,
		size_t *actualBytes, int *errRet, size_t skipBytes = 0);

	virtual OMEevent *selectForRead(OMEeventCallbackFP func,
		OMEthread *t = nullptr) = 0;

	virtual OMEevent *selectForWrite(OMEeventCallbackFP func,
		OMEthread *t = nullptr) = 0;

	virtual int close(int mask = 3) = 0;

	virtual OMEioDescriptor *acceptConnection(int *errRet);

	virtual int receiveDatagram(unsigned char *bfr, size_t bytes,
            size_t *actualBytes, OMEstring *fromInfo, int *errRet);

	virtual int sendDatagram(const unsigned char *bfr, size_t bytes,
            size_t *actualBytes, const char *toInfo, int *errRet);

	virtual int seekToOffset(size_t offset);


	virtual OMEstring *getLocalAddress() const;

	virtual OMEstring *getPeerAddress() const;

	virtual int getErrorStatus() const;

	virtual bool isOpen() const = 0;

	uint_fast16_t adjustVector(OMEioVector *result, uint_fast16_t count,
		const OMEioVector *v, size_t skipCount);
}; // end class OMEioDescriptor

typedef OMEioDescriptor *(*OMEopenSchemeFP)(const OMEstring &url, int *errRet);

int OMEregisterIOscheme(const OMEstring &schemePrefix, OMEopenSchemeFP func);

int OMEregisterIOscheme(const char *schemePrefix, OMEopenSchemeFP func);

OMEioDescriptor *OMEopenURL(const OMEstring &url, int *errRet = nullptr);

/** @}
 */
#endif
/* vim: set expandtab shiftwidth=4 tabstop=4: */
