#ifndef _OME_NAMESPACE_H
//! \cond
#define _OME_NAMESPACE_H "$Id: OMEnamespace.h 357 2020-06-18 23:13:25Z geoff $"
//! \endcond

#include <map>

#include <OMEstring.h>
#include <OMEclass.h>

/** \defgroup OMEnamespace OME Namespace Classes
 * @{
 */

/*!
	\brief Mapping table to class implementations within a namespace.
*/
class OMEnameSpaceGroup {
private:
	OMEstring		nameSpaceName;
	_STD map<uint32_t, class OMEclass *>	classTable;

public:
	explicit OMEnameSpaceGroup(const OMEstring &name) : nameSpaceName(name) {
//		nameSpaceName = name;
	}

	~OMEnameSpaceGroup();

	const OMEstring &getNameSpace() const { return (nameSpaceName); }

	int addClass(OMEclass *rec);

	OME_FAST_CALL OMEclass *findClass(const OMEstring &name, const uint32_t verID = 0) const;
}; // end class OMEnameSpaceGroup

// TO DO:  make static?

OME_FAST_CALL OMEnameSpaceGroup *OMEfindNameSpace(const OMEstring &name, int *errCode = nullptr);

OME_FAST_CALL OMEnameSpaceGroup *OMEdefineNewNameSpace(const OMEstring &name,
	int *errCode = nullptr);



/*!
\brief Create new class entry.
*/
OMEclass *OMEdefineNewClass(const OMEstring &nmSpace, const OMEstring &name,
	const uint32_t verID, const uint32_t iSize, const bool unique = false,
		int *errCode = nullptr,
		const CreateInstanceFP csRoutine = OMEdefaultInstanceCreate,
		const DeleteInstanceFP dsRoutine = OMEdefaultInstanceDelete);

/*!
 * \brief Cover routine that accepts C-style strings.
 */
inline OMEclass *OMEdefineNewClass(const char *nmSpace, const char *name,
	const uint32_t verID, const uint32_t iSize, const bool unique = false,
		int *errCode = nullptr,
		const CreateInstanceFP csRoutine = OMEdefaultInstanceCreate,
		const DeleteInstanceFP dsRoutine = OMEdefaultInstanceDelete)
{
    return (OMEdefineNewClass(OMEstring(nmSpace), OMEstring(name),
                verID, iSize, unique, errCode, csRoutine, dsRoutine));
}



// find an existing class entry, returns 0 if not found
OME_FAST_CALL OMEclass *OMEfindClassDescription(const OMEstring &name, const uint32_t verID = 0,
	const OMEstring *nmSpace = nullptr);

/*! \brief Meta data record to describe an instance variable of a class.
 */
struct OMEinstanceVarDescr {
	const char	*varName;
	uint32_t    declaredType;
	uint32_t	storageType;
};


/*!
\brief Define method implementation.
*/
struct OMEmethodDefinition {
	const char	*nameSpace;
	const char	*className;
	uint32_t	versionID;
	const char	*methodName;
	uint32_t	signature;
	uint32_t	anyMask;
	uint32_t	stackSize;
	uint32_t	invocationMode;
	union {
		NativeMethodFP	function;
		void		*data;
	} code;
	CreateStackFP	createStackRoutine;
	DeleteStackFP	deleteStackRoutine;
	const struct OMEinstanceVarDescr	*threadVariables;
};

// define a method, caching prior results for efficiency
int OMEdefineNewMethod(const OMEmethodDefinition &rec);

OME_FAST_CALL const OMEstring &OMEshareConstant(const OMEstring &stringConstant);

OME_FAST_CALL const OMEstring &OMEshareConstant(OMEstringInROM stringConstant, int precomputedLen = -1);

void OMEclearNameSpaceTable();

/** @}
 */
#endif
/* vim: set expandtab shiftwidth=4 tabstop=4: */
