#ifndef _OME_NLM_DATA_H
//! \cond
#define _OME_NLM_DATA_H "$Id: OMEnlmData.h 312 2020-03-21 21:25:35Z geoff $"
//! \endcond

/*! \file
 * \brief OME native language message catalog implementation.
 */

/** \addtogroup OMEtypes
 * @{
 */

#    include <OMEtype.h>

/*! \brief Interface to OME Native Language Message catalog.
 */
class OME_DLL_EXPORT OMEnlmCatalog
{
private:
    static OMEnlmCatalog *catalogList;
    OMEnlmCatalog *nextEntry;

    OMEstring *findCatalogFile() const;

    int readFile(const OMEstring &fName);

public:
    static OMEnlmCatalog *getCatalog(const OMEstring &catName, const OMEstring &lang);

    OMEstring catalogName;
    OMEstring langID;
    OMEstring *catalogFileName;
    OMEassoc *messageTable;
    uint32_t fileModTime;

    OMEnlmCatalog(const OMEstring &catName, const OMEstring &lang);

    ~OMEnlmCatalog();

    const OMEstring *findMessage(const OMEstring &messId);
}; // end class OMEnlmCatalog

/*! \brief Identifier for an OME Native Language Message.
 */
class OME_DLL_EXPORT OMEnlmData
{
    friend class OMEnlmStorage; // not currently needed
public:
    static void initialize(const char *appName, const char *lang = nullptr);

    OMEstring defaultMessage;
    OMEtype messageID;     // nil => Uniforum style
    OMEstring catalogName; // application name?

    OMEnlmData(const char *dMess, const uint32_t messID = 0,
               const char *catName = nullptr);

    OMEnlmData(const OMEstring &dMess, const OMEtype *messID = nullptr,
               const OMEstring *catName = nullptr);

    ~OMEnlmData() {}

    bool operator==(const OMEnlmData &arg) const;

    bool operator!=(const OMEnlmData &arg) const;

    const OMEstring &getMessage(const OMEstring *lang = nullptr) const;

    void getMessageInfo(OMEarray &list) const
    {
        list[0] = defaultMessage;
        list[1] = messageID;
        list[2] = catalogName;
    }
}; // end class OMEnlmData

/** @}
 */

#endif
/* vim: set expandtab shiftwidth=4 tabstop=4: */
