#ifndef _OME_OIL2_INTERP_H
//! \cond
#define _OME_OIL2_INTERP_H "$Id: OMEoil2Interp.h 452 2020-07-23 20:19:23Z geoff $"
//! \endcond

#include <OMEthread.h>

/** \defgroup OIL2interp OIL2 Interpreter Interfaces
 * @{
 */

/*! \brief Creates a stack for an OIL2 method.
 *
 * \param mInfo describes the method.
 */
extern OME_CORE_CALL void *OMEoil2StackCreate(const OMEinvocationData *mInfo);

/*! \brief Deletes a stack associated with an OIL2 method.
 *
 * \param mInfo describes the method.
 * \param data points to the stack previously created by a call
 * to OMEoil2StackCreate().
 */
extern OME_CORE_CALL void OMEoil2StackDelete(const OMEinvocationData *mInfo, void *data);


//! \cond NOT_USED

/*! \brief Call stack for the OIL2 interpreter.
 */
class OIL2interpreterStack {
public:
    /*! brief Construct the stack for an OIL2 method implemented
     * via Architecture Neutral Format object code.
     *
     * \param iSize specifies the number of instance variables.
     * \param vars describes the instance variables.
     */
	OIL2interpreterStack(const uint_fast32_t iSize, OMEtype &vars);

	~OIL2interpreterStack()	{}
};

//! \endcond

/*! \brief Entry point for the OIL2 Architecture Neutral Format interpreter.
 *
 * \param thread specifies the controlling thread
 * \param codeBody holds the ANF object code.
 */
extern OME_FAST_CALL int OMEoil2Interpreter(OMEthread *thread, const OMEtype &codeBody);


/** @}
 */
#endif
/* vim: set expandtab shiftwidth=4 tabstop=4: */
