#ifndef _OME_PROC_INFO_H
//! \cond
#define _OME_PROC_INFO_H "$Id: OMEprocInfo.h 312 2020-03-21 21:25:35Z geoff $"
//! \endcond

/*! \file
 * \brief OME process information implementation.
 */

/** \addtogroup OMEutilFunctions
 * @{
 */

#include <utils/time/tick_time.h>
#    include <OMEstring.h>

/*! \brief Identifies an instance of an OME process.
 */
class OME_DLL_EXPORT OMEentityID
{
public:
    uint32_t bootTime;
    uint32_t processID;
    uint32_t hostNameHash;

    bool operator==(const OMEentityID &arg) const
    {
        if (memcmp(this, &arg, sizeof(OMEentityID)) == 0) {
            return (true);
        }
        return (false);
    }

    bool operator!=(const OMEentityID &arg) const
    {
        if (memcmp(this, &arg, sizeof(OMEentityID)) == 0) {
            return (false);
        }
        return (true);
    }
    bool operator<(const OMEentityID &arg) const
    {
        if (memcmp(this, &arg, sizeof(OMEentityID)) < 0) {
            return (true);
        }
        return (false);
    }

    template <typename STREAMTYPE> STREAMTYPE &outputOnStream(STREAMTYPE &outputStream, int_fast16_t indent = 0) const
    {
        outputStream << "(" << bootTime << "|" << processID << "|" << hostNameHash << ")";
        return (outputStream);
    }
}; // end class OMEentityID

template <typename STREAMTYPE> inline STREAMTYPE &operator<<(STREAMTYPE &outputStream, const OMEentityID &data)
{
    data.outputOnStream(outputStream, 0);
    return (outputStream);
}

/*! \brief Uniquely identifies an OME process.
 */
class OME_DLL_EXPORT OMEthisProcessInfo
{
public:
    OMEstring hostName;
    OMEentityID id;

    OMEthisProcessInfo();

    ~OMEthisProcessInfo() {}
};

extern OME_DLL_EXPORT OMEthisProcessInfo OMEcurrentProcessInfo;

extern OME_DLL_EXPORT OMEstring *OMEgetUserName();

// uses implementation from utils/tick_time.h
#define OMEgetCPUticks() get_CPU_ticks()

extern OME_DLL_EXPORT int64_t OMErelativeMilliseconds();
extern OME_DLL_EXPORT int64_t OMEmillisecondsSinceBoot();
extern OME_DLL_EXPORT int64_t OMEticksSinceBoot();

/** @}
 */

#endif
/* vim: set expandtab shiftwidth=4 tabstop=4: */
