#ifndef _OME_RUNTIME_H
//! \cond
#define _OME_RUNTIME_H "$Id: OMEruntime.h 453 2020-07-23 20:20:44Z geoff $"
//! \endcond

// single global include for applications

#include <OMEdebugInfo.h>
#include <OMEnamespace.h>
#include <OMEthread.h>
#include <OMEcore.h>

/** \defgroup OMEmain OME Processing Loop Routines
 * @{
 */

/*! \brief Library main entry point.
 */
int VISTAOMEmain(int argc, const char **argv, const char **envp = nullptr);

/*!
\brief Standard initialization routine for FARGOS/VISTA
Object Management Environment.
\param rcFileName identifies the name of an boot script file to be processed
via CreateObjects.
\param argc is a count of the arguments in the \a argv array.
\param argv is an array of null-terminated C strings, corresponding
to the arguments to the program.  Subscript 0 holds the name used to
invoke the program.
\param envp is an array of null-terminated C string, which hold the
initial set of process environment variables.

The OMEinitSystem() function calls various initialization routines and
then creates the special ObjectCreator object.  To complete the boot process,
a CreateObjects object is created and provided the \a rcFileName argument.

The actual execution of the system does not take place within this
initialization routine.  Instead, OMEmainLoop() or an equivalent must be called
to enable the execution of method invocations.
\retval 0 indicates successful initialization.
\retval -1 indicates unsuccessful initialization.

\sa OMEinitDebugFlag
\sa OMEnlmData::initialize()
\sa OMEsetDebugFlag()
\sa OMEgetSystemUser()
\sa OMEthread::defineOMEthreadClass()
\sa OMEobjectInstance::defineOMEobjectClass()
\sa OMEdisplayModuleComments() 
*/
int OMEinitSystem(const char *rcFileName,
	int argc = 0, const char **argv = nullptr, const char **envp = nullptr);

/*!
\brief Run method invocations and handle events, such as input available
and timer expiration.
*/
int OMEmainLoop(uint_fast32_t methodLimit = 20000);

/** @}
 */
#endif
/* vim: set expandtab shiftwidth=4 tabstop=4: */
