#ifndef _OME_TYPE_H
#    define _OME_TYPE_H "$Id: OMEtype.h 312 2020-03-21 21:25:35Z geoff $"

/*! \file
 * \brief OME fundamental type implementation.
 */
/** \defgroup OMEtypes OME Type Support
 * @{
 */

#    include <OMEmanifests.h>
#    include <OMEarray.h>
#    include <OMEassoc.h>
#    include <OMEoid.h>
#    include <OMEset.h>
#    include <OMEstring.h>
#    include <OMEfixed.h>
#    include <OMEnlm.h>

/*
#define OME_NIL     0
#define OME_INT     1
#define OME_INT32   1
#define OME_INT64   2
#define OME_FLOAT   3
#define OME_DOUBLE  4
#define OME_OID     5
#define OME_STRING  6
#define OME_ARRAY   7
#define OME_ASSOC   8
// 9 NOT YET USED
#define OME_SET     10
#define OME_NLM     11
#define OME_FIXED   12
#define OME_POINTER 13
#define OME_ANY     14
*/

#    define OME_SIMPLE_TYPE 1    //!< A simple native type
#    define OME_EXTENDED_TYPE 2  //!< An extended native type
#    define OME_REFERENCE_TYPE 3 //!< A complex, reference counted type

/*! \brief Internal structure to describe OME fundamental types.
 */
struct _OMEtypeDescription {
    uint8_t typeClass;
    uint8_t basicType;  // int, float
    char    typeName[17];
    char    enumName[12];
};

extern const _OMEtypeDescription _OMEtypeTable[]; // hidden type table

#define OMEtypeClass(t) (_OMEtypeTable[(t) & 0x1f].typeClass)
#define OMEfundamentalType(t) (_OMEtypeTable[(t) & 0x1f].basicType)

typedef class OMEbaseType *OMEbaseTypeP;
typedef const class OMEbaseType &READ_ONLY_OMEtype;



#include <OMEbaseType.h>

#include <OMEoutputOnStream.h>

#    if 0
// simple chained operators...
#    include <OMEtypeHdr3.genh>
#    include <OMEtypeHdr4.genh>

uint32_t operator+(const uint32_t, const OMEbaseType &);
uint32_t operator+(const float, const OMEbaseType &);
uint32_t operator+(const OMEbaseType &, const uint32_t);
uint32_t operator-(const uint32_t, const OMEbaseType &);
uint32_t operator-(const float, const OMEbaseType &);
uint32_t operator-(const OMEbaseType &, const uint32_t);
uint32_t operator*(const uint32_t, const OMEbaseType &);
uint32_t operator*(const float, const OMEbaseType &);
uint32_t operator*(const OMEbaseType &, const uint32_t);
uint32_t operator/(const uint32_t, const OMEbaseType &);
uint32_t operator/(const float, const OMEbaseType &);
uint32_t operator/(const OMEbaseType &, const uint32_t);

uint32_t operator%(const OMEbaseType &, const uint32_t);

uint32_t operator&(const uint32_t, const OMEbaseType &);
uint32_t operator&(const OMEbaseType &, const uint32_t);
uint32_t operator|(const uint32_t, const OMEbaseType &);
uint32_t operator|(const OMEbaseType &, const uint32_t);
uint32_t operator^(const uint32_t, const OMEbaseType &);
uint32_t operator^(const OMEbaseType &, const uint32_t);


bool operator==(const int32_t, const OMEbaseType &);
bool operator==(const uint32_t, const OMEbaseType &);
bool operator!=(const int32_t, const OMEbaseType &);
bool operator!=(const uint32_t, const OMEbaseType &);
bool operator<=(const int32_t, const OMEbaseType &);
bool operator<=(const uint32_t, const OMEbaseType &);
bool operator>=(const int32_t, const OMEbaseType &);
bool operator>=(const uint32_t, const OMEbaseType &);
bool operator<(const int32_t, const OMEbaseType &);
bool operator<(const uint32_t, const OMEbaseType &);
bool operator>(const int32_t, const OMEbaseType &);
bool operator>(const uint32_t, const OMEbaseType &);
#    endif


OME_FAST_CALL int stringToArrayOfTokens(OMEtype &result, const OMEstring &source,
                                        const char *delimeterList = nullptr, const int convertFlag = 0);

OME_FAST_CALL int stringToArrayOfTokens(OMEtype &result,
                                        const char *source, const int srcLen,
                                        const char *delimeterList = nullptr, const int convertFlag = 0,
                                        const int charSet = OME_CHARSET_UNDEFINED);

OME_FAST_CALL void OMEconvertString(OMEtype &result, OMEstring *source,
                                    const uint_fast16_t forceBase = 10,
                                    const bool discard = false);

#    define OME_FLAG_SORT_ASCENDING 0        //!< Sort ascending
#    define OME_FLAG_SORT_DESCENDING 1       //!< Sort descending
#    define OME_FLAG_SORT_BY_KEY 2       //!< Sort by associative array key
#    define OME_FLAG_SORT_CASE_INSENSITIVE 4 //!< Ignore case during sort

OME_FAST_CALL int subscriptsInOrder(OMEtype &result, const OMEtype &list, uint_fast8_t descending = OME_FLAG_SORT_ASCENDING);


extern int OMEabortOnError; //!< If non-zero, abort() called on type error.

/** @}
 */

#endif

/* vim: set expandtab shiftwidth=4 tabstop=4: */
