#ifndef _CIRCULAR_WAIT_HPP_
//! \cond
#define _CIRCULAR_WAIT_HPP_ "$Id: circular_wait.hpp 316 2020-03-26 16:18:40Z geoff $"
//! \endcond

/** \addtogroup FARGOSbfr
 * @{
 */

/* Implementation of waitForBufferAllocRecord()
 * so that it can be inlined on non-Windows hosts
 * or deferred so as to avoid the need for windows.h
 * and seeing it prior to winsock2.h
 */

void waitForBufferAllocRecordToBeReady(SharedBufferAllocRecord *rec)
{
    if (OME_EXPECT_FALSE(rec->usedLen == ~0U)) {
        uint_fast32_t spinCount = 0;
        do {
            spinCount += 1;
            if ((spinCount & 0x7) == 0) {
#ifndef _WIN32
                struct timespec now;
                now.tv_sec = 0;
                now.tv_nsec = 500000; // delay 500 microseconds
                nanosleep(&now, nullptr);
#else
                SleepEx(1, FALSE); // delay 1 millisecond
#endif
            } else {
                OME_YIELD_THREAD();
            }
        } while (rec->usedLen == ~0U);
    }
}

/** @} */
#endif
/* vim: set expandtab shiftwidth=4 tabstop=4: */
