#ifndef _GET_TIME_H_
//! \cond
#define _GET_TIME_H_ "$Id: get_time.h 315 2020-03-26 16:17:02Z geoff $"
//! \endcond

/* Copyright (C) 2010 - 2019 FARGOS Development, LLC */

#include <stdint.h>

/*! \file
 * \brief Time acquisition routines.
 */
/** \addtogroup FARGOStime
 * @{
 */

#ifdef __cplusplus
extern "C" {
#endif

/*! \brief Structure for representing time as relative seconds and
 * nanoseconds.
 */
struct TimeWithNanoseconds {
    uint64_t    time_sec;       /* 64-bits for all-platform portability */
    uint64_t    time_nanosec;
};

/*! \brief Structure for representing day-of-the-year. */
struct TimeAsDayAndYear {
    uint16_t        year;
    unsigned char   month;
    unsigned char   day;
};

/*! \brief Structure for representing time of day */
struct TimeAsHourMinSec {
    unsigned char   hour;
    unsigned char   min;
    unsigned char   sec;
    unsigned char   _fill1;
};

#ifdef __cplusplus
#define _GCT_DEFAULT_ARG = 0
#else
#define _GCT_DEFAULT_ARG
#endif

/*! \brief Get the current time
 *
 * \param nowTime points to a TimeWithNanoseconds structure into which a copy
 * of the current time will be saved.  If possible, the time is determined
 * via interpolation of the CPU's current tick register value.
 * \param force is a Boolean flag that indicates if interpolation should
 * not be attempted and instead a kernel call made to obtain the current
 * time.
 */
const struct TimeWithNanoseconds *getCurrentTime(
        struct TimeWithNanoseconds *nowTime /*! \cond */ _GCT_DEFAULT_ARG /*! \endcond */ ,
        bool force /*! \cond */ _GCT_DEFAULT_ARG /*! \endcond */);

/*! \brief Get today's date
 *
 * \param nowTimeFields points to a TimeAsHourMinSec structure into which
 * the current local time will be stored.
 *
 * \return A pointer to a TimeAsDayAndYear structure populated with
 * today's date is returned.
 */
const struct TimeAsDayAndYear *getTodaysDate(struct TimeAsHourMinSec *nowTimeFields /*! \cond */ _GCT_DEFAULT_ARG /*! \endcond */);

#ifdef __cplusplus
#undef _GCT_DEFAULT_ARG
#endif

#ifdef __cplusplus
}; /* close extern "C" scope */
#endif

/** @} */
#endif
/* vim: set expandtab shiftwidth=4 tabstop=4: */
